@@ -1,13 +1,12 @@
 ORGANISM <- "Xenopus tropicalis"
 
-### List of assemblies by date
+### List of assemblies by date.
 ASSEMBLIES <- list(
-    ## strain: Nigerian
     list(assembly="UCB_Xtro_10.0",
          assembly_level="Chromosome",
          date="2019/11/14",
          extra_info=c(strain="Nigerian"),
-         assembly_accession="GCF_000004195.4",
+         assembly_accession="GCF_000004195.4",  # xenTro10
          circ_seqs="MT"),
 
     list(assembly="ASM1336827v1",
@@ -17,3 +16,4 @@ ASSEMBLIES <- list(
          assembly_accession="GCA_013368275.1",
          circ_seqs=character(0))
 )
+
